#include <stdio.h>
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/StripChart.h>
#include <X11/Xaw/MenuButton.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>

#define	Xt_W	XtVaCreateManagedWidget

#define	DEFAULT_INTERVAL	60
#define	DEV_MAX			16

#ifdef __NetBSD__
# define PS_NAME		"/usr/sbin/pstat -s"
# define DF_NAME		"/bin/df"
# define FORMAT_MAX	6
# define DF_NAME_PTR	0
# define DF_ALL_PTR	1
# define DF_FRE_PTR	3
/*                                                     dev  all  use  ave  cap  typ */
# define PSscanf(A,C)	sscanf(A,"%s %s %s %s %s %s\n",C[0],C[1],C[2],C[3],C[4],C[5])
# define DFscanf(A,C)	sscanf(A,"%s %s %s %s %s %s\n",C[0],C[1],C[2],C[3],C[4],C[5])
#endif

#ifdef __sun__
# define PS_NAME		"/usr/etc/pstat -s"
# define DF_NAME		"/bin/df"
# define FORMAT_MAX	10
# define PS_ALL_PTR	8
# define PS_FRE_PTR	6
# define DF_NAME_PTR	0
# define DF_ALL_PTR	1
# define DF_FRE_PTR	3
/*                                                                   0k alloc    +   0k  res    =   0k used   0k avail */
# define PSscanf(A,C)	sscanf(A,"%s %s %s %s %s %s %s %s %s %s\n",C[0],C[1],C[2],C[3],C[4],C[5],C[6],C[7],C[8],C[9])
# define DFscanf(A,C)	sscanf(A,"%s %s %s %s %s %s\n"            ,C[0],C[1],C[2],C[3],C[4],C[5])
#endif

#define	STRING_MAX	256

Widget top,base,chart,button,menu,entry;
static char devname[DEV_MAX][STRING_MAX];
static int devno = 0;

void main(int argc,char **argv)
{
	void GetValue();
	void DevChange();
	int GetDevName();
	int i;
	int devnum;
	int interval;

	devnum = GetDevName();

	top   = XtInitialize(argv[0],"Xfreechart",NULL,0,&argc,argv);

	interval = DEFAULT_INTERVAL;
	if (argc >= 2) {
		if ((interval = atoi(argv[1])) <= 0)
			interval = DEFAULT_INTERVAL;
	}

	base   = Xt_W("base"  ,formWidgetClass,top,NULL);

	button = Xt_W("button",menuButtonWidgetClass,base,
		      XtNlabel,devname[devno],
		      NULL);

	menu   = XtCreatePopupShell("menu",simpleMenuWidgetClass,button,
				    NULL,0);

	for(i=0;i<devnum;i++) {
		char str[7];
		sprintf(str,"item%d",i);
		entry = Xt_W(str,smeBSBObjectClass,menu,
			     XtNlabel,devname[i],
			     NULL);
		XtAddCallback(entry,XtNcallback,DevChange,i);
	}

	chart  = Xt_W("chart" ,stripChartWidgetClass,base,
		      XtNupdate,interval,	/* sec */
		      XtNjumpScroll,1,
		      XtNwidth,80,
		      XtNheight,80,
		      XtNfromVert,button,
		      NULL);
	XtAddCallback(chart,XtNgetValue,GetValue,NULL);

	XtRealizeWidget(top);
	XtMainLoop();
}

void DevChange(Widget w,int no,caddr_t call_data)
{
	XtVaSetValues(button,XtNlabel,devname[no],NULL);
	devno = no;
}

void GetValue(Widget w,caddr_t client_data,double *value)
{
	FILE *fp;
	char    str[STRING_MAX];
	char	f[FORMAT_MAX][STRING_MAX];
	char	*name;
	char	*fre_val;
	char	*all_val;
	static int fre = 0;
	static int all = 0;

	if (devno == 0) {
		fp = popen(PS_NAME,"r");
		if (fp) {
			while(!feof(fp)) {
				if (fgets(str,STRING_MAX,fp)) {
					PSscanf(str,f);
					all_val = f[PS_ALL_PTR];  all_val[strlen(all_val)] = '\0';
					fre_val = f[PS_FRE_PTR];  fre_val[strlen(fre_val)] = '\0';
#ifdef __NetBSD__
					if (!strncmp(str,"/dev",4)) {
#endif
#ifdef __sun__
					if (!strcmp(f[1],"allocated")) {
#endif
						all = atoi(all_val);
						fre = atoi(fre_val);
						break;
					}
				}
			}
		}
	}
	else {
		fp = popen(DF_NAME,"r");
		if (fp) {
			while(!feof(fp)) {
				if (fgets(str,STRING_MAX,fp)) {
					DFscanf(str,f);
					name    = f[DF_NAME_PTR];
					all_val = f[DF_ALL_PTR];  all_val[strlen(all_val)] = '\0';
					fre_val = f[DF_FRE_PTR];  fre_val[strlen(fre_val)] = '\0';
					if ((devno != 0)&&(!strcmp(devname[devno],name))) {
						all = atoi(all_val);
						fre = atoi(fre_val);
						break;
					}
				}
			}
		}
	}
	pclose(fp);
	*value = (double)fre/(double)all;
}

int GetDevName(void)
{
	FILE *fp;
	char *name;
	char str[STRING_MAX];
	char f[FORMAT_MAX][STRING_MAX];
	int num = 0;

	strcpy(devname[num++],"Memory   ");

	fp = popen(DF_NAME,"r");
	if (fp) {
		while(!feof(fp)) {
			if (fgets(str,STRING_MAX,fp)) {
				PSscanf(str,f);
				name = f[DF_NAME_PTR];

				name[strlen(name)] = '\0';
				if (!strncmp(name,"/dev",4))
					strcpy(devname[num++],name);
				if (num >= DEV_MAX)
					break;
			}
		}
	}
	pclose(fp);

	return num;
}
